from __future__ import division

from vector import *

class Bezier(object):
    def __init__(self, p0, p1, p2, p3):
        self.p0 = p0
        self.p1 = p1
        self.p2 = p2
        self.p3 = p3
        
        self.coeffs = [complex(*x) for x in
                        [ p0,
                        3*p1 - 3*p0,
                        3*p2 - 6*p1 + 3*p0,
                        p3 - 3*p2 + 3*p1 - p0 ]]
        self.center = .25 * (p0 + p1 + p2 + p3)
        self.radius = max((p0 - self.center).length, (p1 - self.center).length, (p2 - self.center).length, (p3 - self.center).length)
            
    def pos(self, t):
        p = self.coeffs[0] + t * (self.coeffs[1] + t * (self.coeffs[2] + t * self.coeffs[3]))
        return Vector((p.real, p.imag))
    def dpos(self, t):
        p = (3*t**2)*self.coeffs[3] + (2*t)*self.coeffs[2] + self.coeffs[1]
        return Vector((p.real, p.imag))
        
    def tangent(self, t):
        p = (3*t**2)*self.coeffs[3] + (2*t)*self.coeffs[2] + self.coeffs[1]
        p /= abs(p)
        return Vector((p.real, p.imag))
    
    def normal(self, t):
        p = (3*t**2)*self.coeffs[3] + (2*t)*self.coeffs[2] + self.coeffs[1]
        p /= -1j * abs(p)
        return Vector((p.real, p.imag))
                
    def d2pos(self, t):
        p = 2 * (3*t*self.coeffs[3] + self.coeffs[2])
        return Vector((p.real, p.imag))

    def dist_sq(self, t, r):        
        return (self.pos(t) - r).length2
        
    def ddist_sq(self, t, r):
        return 2*self.dpos(t).dot(self.pos(t) - r)
        
    def d2dist_sq(self, t, r):
        return 2*(self.dpos(t).length2 + self.d2pos(t).dot(self.pos(t) - r))
    
    def curvature(self, t):
        d = self.dpos(t)
        return d.dot(self.d2pos(t)) / (d.length ** 3)
    # omg
    
    def closest(self, r, t):
        sc0, sc1, sc2, sc3 = self.coeffs
        r = complex(*r)
        p = sc0 + t * (sc1 + t * (sc2 + t * sc3)) - r
        dp = (3*t*sc3 + 2*sc2) * t + sc1
        d2p = 2 * (3*t*sc3 + sc2)
        p_ = p.conjugate()
        t -= (dp * p_).real / (abs(dp) ** 2 + (d2p * p_).real)
        if t < 0: return 0
        if t > 1: return 1
        return t
