from __future__ import division

import sys

import pyglet

import constants
constants.DEBUG = True

import config
import hiscores
import model.race
import presidents
import rpc.client
import season
import ctypes
from common import *
from constants import *


def validate_score(hs_tuple):
    racemode, username, racename, ticks, timestamp, encodedghost = hs_tuple

    ghostdata = hiscores.decode_ghost(encodedghost)
    file("ghost.data", "w").write(repr(ghostdata))
    if racemode in ('GP', 'TT'):
        s = None
    else:
        s = season.Season(racemode, single=True)

    r = model.race.Race(racename, racemode, 'watch', ghostdata, season=s)

    while not r.is_victorious():
        # if r.current_tick in [218, 219]:
        #             for e in r.entities:
        #                 if e.__class__.__name__ == 'LoveLoveRacer':
        #                     print e, raw_values(e.old_pos), raw_values(e.pos)
        r.tick()
        if r.current_tick * 100 > ticks: break
    
    return (r.player_finish_time == ticks)

class Flag(object):
    response = None
    def on_rpc_response(self, request, response):
        self.response = response
    def on_rpc_error(self, request, error):
        raise error

def all_tracks():
    for mode in ('GP', 'TT'):
       for race in presidents.order:
           yield (mode, race)
    for mode in presidents.seasonorder:
       for race in presidents.seasons[mode]:
           yield (mode, race)

if __name__ == '__main__':
    rpc = rpc.client.RPCManager(config.rpchost, config.rpcport, RPC_VERSION, debug=RPC_DEBUG, timeout=RPC_TIMEOUT)
    fh_succ = open('check-succ.txt', 'w')
    fh_fail = open('check-fail.txt', 'w')

    for mode, race in all_tracks():
        print "Retrieving best times for %s: %s" % (mode, race)
        
        req = rpc.get_hiscores(mode, race)
        flag = Flag()
        req.listen(flag)
        while flag.response is None:
            rpc.dispatch_events()
            
        for hsobj in flag.response:
            description = "%s: %s by %s - %d at %d" % (hsobj[0], hsobj[2], hsobj[1], hsobj[3], hsobj[4])
            print description + ' ...',
            sys.stdout.flush()
            ok = validate_score(hsobj)
            if ok:
                print "OK"
                fh_succ.write(description + '\n')
                fh_succ.flush()
            else:
                print "FAIL"
                fh_fail.write(description + '\n')
                fh_fail.flush()
    
    fh_succ.close()
    fh_fail.close()