"""Constant definitions.

This file is loaded during the program initialisation before the main module is
imported. Consequently, this file must not import any other modules, or those
modules will be initialised before the main module, which means that the DEBUG
value may not have been set correctly.

This module is intended to be imported with 'from ... import *' semantics but
it does not provide an __all__ specification.

"""

#: Enable debug features. Should never be changed manually but is set to True
#: automatically when running `test_game.py`.
DEBUG = False

#: Version string. This may be represented somewhere in-game. It is also read
#: by `setup.py` as part of its versioning features.
VERSION = u"1.0.4"

#: Version specifications used by RPC client/server.
RPC_VERSION = [1, 0]

#: Compatibility version specification used by RPC client/server.
RPC_COMPAT = [1, 0]

#: Debug flag used by RPC client/server.
RPC_DEBUG = True

#: The number of seconds until RPC requests timeout.
RPC_TIMEOUT = 10

#: The directory (relative to the top level) wherein all the resources for the
#: game are stored, probably subdivided into types of resources. See `data.py`.
DATA_DIR = "data"

#: The name of the game used in locating the saved settings directory. Its best
#: not to have any spaces in this name.
CONFIG_NAME = "SuperEffective10"

#: The caption that appears at the top of the window. Obviously this is only
#: visible in windowed mode.
CAPTION = u"Intestinvaders!"

#: The "top-level" tick rate; the maximum number of times per second that the
#: controller will call its tick method.
TICK_RATE = 60.0

#: The "top-level" update rate; the maximum number of times per second that the
#: controller will call its update method.
UPDATE_RATE = 60.0

TRACK_SEGMENTS = 30
SPLINE_RESOLUTION = 15
WORLD_WIDTH = 1000
WORLD_HEIGHT = 750
PLAYER_THRUST = 0.5
PLAYER_ROTATION = 3
MAX_SCORES = 5
WALL_SECTIONS = 1

BUMP_MAGNITUDE = 2

## Interface
############

FONT_NAME = "Anamorphic"
LINK_NORMAL = (255, 255, 255, 255)
LINK_HILIGHT = (255, 255, 0, 255)
