from __future__ import division

import math
import os
import re
import random
import textwrap

import pyglet
from pyglet.gl import *

import data
import hiscores
import interface
import mode
import fx
import presidents
import season
import squirtle
import sound
from vector import v

import config
from common import *
from constants import *

intro_text = re.sub(ur"(?<!\n)\n(?!\n)", ur" ", u"""\
The year is 2391.

Earth is a utopia.

Mankind's technological achievements have brought about the rise of the most
exciting sport ever created: tiny miniature spaceships travel back in time, in
order to race through the intestinal tracts of unsuspecting inhabitants of the
previous centuries.

The most accomplished and daring racers are selected to participate in the
World Championships, a series of races which take place inside the most
important colons in history, those belonging to the most powerful men in the
world: The Presidents of the United States of America!

Do you have the guts to become an INTESTINVADER?
""").strip()

def surpress_text(window):
    def on_text(text):
        window.remove_handlers(on_text)
        return pyglet.event.EVENT_HANDLED
    window.push_handlers(on_text)

def make_labels(size, batch, texts):
    _tls = []
    for pres in texts:
        _tls.append((pres, pyglet.text.Label(text=pres, font_name=FONT_NAME,
                font_size=size, batch=batch)))
    return _tls

class Label(interface.Interface):
    _kwds = {"label" : None}
    _no_event = True
    def _content_size(self):
        return self._label.content_width, self._label.content_height
    def draw_content(self):
        self._label.draw()
    def set_mouse(self, x, y):
        pass

class Tabel(interface.Tabular):
    _kwds = {"font_size" : 0.08, "callback" : staticmethod(lambda t: None),
            "texts": ()}
    _scaled = [(), ("font_size",),]
    _batch = None
    def set_texts(self, texts):
        if texts != self._texts:
            self._texts = texts
            self._clean = False
            for x in self._cached:
                delattr(self, x)
    def clean_content(self):
        for c in self._components:
            c._label.delete()
        self._components = []
        self._batch = pyglet.graphics.Batch()
        labels = make_labels(self.real_font_size, self._batch, self.texts)
        ls = [Label(label=l, layout=self) for n, l in labels]
    def organise(self):
        super(Tabel, self).organise()
        ls = self._components
        self._offsets = [l.content_offset for l in ls]
        self._sizes = [l.content_size for l in ls]
        for idx, label in enumerate(self._components):
            label._label.x = self._offsets[idx][0]
            label._label.y = self._offsets[idx][1]
    def set_mouse(self, x, y):
        self._hl = None
        pxoff, pyoff = self.parent_offset
        xoff, yoff = self.content_offset
        xoff, yoff = xoff + pxoff, yoff + pyoff
        for idx, label in enumerate(self._components):
            lxoff = xoff + self._offsets[idx][0]
            lyoff = yoff + self._offsets[idx][1]
            width, height = self._sizes[idx]
            if lxoff < x < lxoff + width and lyoff < y < lyoff + height:
                c = LINK_HILIGHT
                self._hl = label
            else:
                c = LINK_NORMAL
            if label._label.color != c:
                label._label.color = c
        super(Tabel, self).set_mouse(x, y)
    def click(self):
        if self._hl is not None:
            self.callback(self._hl._label.text)
            return True
    def draw(self):
        if self.show:
            self.make_clean()
            self.draw_box()
            glPushMatrix()
            xoff, yoff = self.content_offset
            w, h = self.content_size
            glTranslatef(xoff + w / 2, yoff + h / 2, 0.0)
            glRotatef(self.aoff, 0.0, 0.0, 1.0)
            glTranslatef(-w / 2, -h / 2, 0.0)
            self._batch.draw()
            self.draw_content()
            glPopMatrix()

class Ship(interface.Image):
    def set_mouse(self, x, y):
        pxoff, pyoff = self.parent_offset
        xoff, yoff = self.content_offset
        xoff, yoff = xoff + pxoff, yoff + pyoff
        width, height = self.content_size
        xoff += width / 2
        yoff += height / 2
        r = max(width, height) / 2
        self._mouse_in_interface = (x - xoff) ** 2 + (y - yoff) ** 2 < r ** 2

class MenuModelState(object):

    def __init__(self):
        if config.username is None:
            self.set_username()
        else:
            self.set_main()
        self.hiscore_scope = "personal"
        self.hiscore_mode = "Grand Prix"
        self.hiscore_track_name = "George Washington"
        self.hiscore_season_name = "Founding Fathers"
        self.track_name = "George Washington"
        self.season_track_name = "Founding Fathers"
        self.ship_name = config.ship
        self.race_mode = "Grand Prix"
        self._old = {}
        
    def copy(self):
        new = MenuModelState()
        new.__dict__.update(self.__dict__)
        return new

    def clear(self):
        self._old = dict(self.__dict__)
        self.show_title = True
        self.show_start = False
        self.show_season = False
        self.show_username = False
        self.show_intro = False
        self.show_main = False
        self.show_options = False
        self.show_hiscores = False
        self.show_track_choose = False
        self.show_ship_choose = False
        self.show_season_choose = False
        self.exiting = False
        self.starting = False

    def set_username(self):
        self.clear()
        self.show_title = False
        self.show_username = True

    def set_intro(self):
        self.clear()
        self.show_title = False
        self.show_intro = True

    def set_main(self):
        self.clear()
        self.show_main = True

    def set_start(self):
        self.clear()
        self.show_start = True

    def set_season(self):
        self.clear()
        self.show_season = True

    def set_options(self):
        self.clear()
        self.show_options = True

    def set_hiscores(self):
        self.clear()
        self.show_hiscores = True
        self.show_title = False

    def set_exit(self):
        self.clear()
        self.show_title = False
        self.exiting = True

    def set_starting(self):
        self.set_exit()
        self.starting = True

    def set_back(self):
        self.__dict__.update(self._old)

    def toggle_track_choose(self):
        self.show_track_choose = not self.show_track_choose
        if self.show_season_choose and self.show_season_choose:
            self.show_season_choose = False

    def toggle_season_choose(self):
        self.show_season_choose = not self.show_season_choose
        if self.show_season_choose and self.show_season_choose:
            self.show_track_choose = False

    def set_ship_choose(self):
        self.clear()
        self.show_ship_choose = True


class MenuViewState(object):

    def __init__(self, handler, model_state):
        self.handler = handler
        self.model_state = self.ms = model_state
        self.title_offset = 1.0
        self.username_offset = 1.0
        self.intro_offset = 1.0
        self.main_offset = 1.0
        self.start_offset = 1.0
        self.season_offset = 1.0
        self.options_offset = 1.0
        self.hiscores_offset = 1.0
        self.track_choose_offset = 1.0
        self.season_choose_offset = 1.0
        self.ship_choose_offset = 1.0
        self.track_name_offset = 1.0
        self.ship_popup = 0.0
        self.hsseason_offset = 0.0
        self.hsseason_fade = 1.0
        self.fade_out = 1.0
        self.wait_call = []

    @property
    def show_title(self):
        return 0.0 <= self.title_offset < 1.0

    @property
    def show_username(self):
        return 0.0 <= self.username_offset < 1.0

    @property
    def show_intro(self):
        return 0.0 <= self.intro_offset < 1.0

    @property
    def show_main(self):
        return 0.0 <= self.main_offset < 1.0

    @property
    def show_start(self):
        return -1.0 < self.start_offset < 1.0

    @property
    def show_season(self):
        return -1.0 < self.season_offset < 1.0

    @property
    def show_options(self):
        return 0.0 <= self.options_offset < 1.0

    @property
    def show_hiscores(self):
        return 0.0 <= self.hiscores_offset < 1.0

    @property
    def show_track_choose(self):
        return 0.0 <= self.track_choose_offset < 1.0

    @property
    def show_season_choose(self):
        return 0.0 <= self.season_choose_offset < 1.0

    @property
    def show_ship_choose(self):
        return 0.0 <= self.ship_choose_offset < 1.0

    @property
    def show_hsseason(self):
        return 0.0 < self.hsseason_fade <= 1.0

    def _aim_for(self, attr, target, speed, reverse=False):
        value = getattr(self, attr)
        if value < target: setattr(self, attr, min(target, value + speed))
        elif value > target: setattr(self, attr, max(target, value - speed))
        else: return False
        return True

    def tick_title(self):
        if self.ms.show_title:
            return self._aim_for("title_offset", 0.0, 0.02)
        return self._aim_for("title_offset", 1.0, 0.02)

    def tick_username(self):
        if self.ms.show_username:
            return self._aim_for("username_offset", 0.0, 0.02)
        return self._aim_for("username_offset", 1.0, 0.02)

    def tick_intro(self):
        if self.ms.show_intro:
            return self._aim_for("intro_offset", 0.0, 0.02)
        return self._aim_for("intro_offset", 1.0, 0.02)

    def tick_main(self):
        if self.ms.show_main:
            return self._aim_for("main_offset", 0.0, 0.02)
        return self._aim_for("main_offset", 1.0, 0.02, self.ms.exiting)

    def tick_start(self):
        if self.ms.show_start:
            return self._aim_for("start_offset", 0.0, 0.02)
        return self._aim_for("start_offset", 1.0, 0.02)

    def tick_season(self):
        if self.ms.show_season:
            return self._aim_for("season_offset", 0.0, 0.02)
        return self._aim_for("season_offset", 1.0, 0.02)

    def tick_options(self):
        if self.ms.show_options:
            return self._aim_for("options_offset", 0.0, 0.02)
        return self._aim_for("options_offset", 1.0, 0.02)

    def tick_hiscores(self):
        if self.ms.show_hiscores:
            return self._aim_for("hiscores_offset", 0.0, 0.02)
        return self._aim_for("hiscores_offset", 1.0, 0.02)

    def tick_track_choose(self):
        if self.ms.show_track_choose:
            return self._aim_for("track_choose_offset", 0.0, 0.02)
        return self._aim_for("track_choose_offset", 1.0, 0.02)

    def tick_season_choose(self):
        if self.ms.show_season_choose:
            return self._aim_for("season_choose_offset", 0.0, 0.02)
        return self._aim_for("season_choose_offset", 1.0, 0.02)

    def tick_ship_choose(self):
        if self.ms.show_ship_choose:
            return self._aim_for("ship_choose_offset", 0.0, 0.02)
        return self._aim_for("ship_choose_offset", 1.0, 0.02)

    def tick_track_name(self):
        if self.ms.track_name != self.handler.track_name.text:
            return self._aim_for("track_name_offset", 1.0, 0.02)
        return self._aim_for("track_name_offset", 0.0, 0.02)

    def tick_ship(self):
        if self.handler.ship._mouse_in_interface:
            self._aim_for("ship_popup", 1.0, 0.08)
        else:
            self._aim_for("ship_popup", 0.0, 0.08)
        return False

    def tick_fade(self):
        if self.ms.exiting:
            return self._aim_for("fade_out", 1.0, 0.02)
        return self._aim_for("fade_out", 0.0, 0.02)

    def tick_hsseason(self):
        if self.ms.hiscore_mode != u"Season":
            if not self._aim_for("hsseason_fade", 0.0, 0.02):
                return self._aim_for("hsseason_offset", 1.0, 0.02)
            return True
        if not self._aim_for("hsseason_offset", 0.0, 0.02):
            return self._aim_for("hsseason_fade", 1.0, 0.02)
        return True

    def wait_animation(self, func, *args, **kwds):
        self.wait_call.append((func, args, kwds))

    def tick(self):
        animating = self.tick_title()         | \
                    self.tick_username()      | \
                    self.tick_intro()         | \
                    self.tick_main()          | \
                    self.tick_start()         | \
                    self.tick_season()        | \
                    self.tick_options()       | \
                    self.tick_hiscores()      | \
                    self.tick_track_choose()  | \
                    self.tick_season_choose() | \
                    self.tick_ship_choose()   | \
                    self.tick_ship()          | \
                    self.tick_hsseason()      | \
                    self.tick_fade()
        if not animating:
            while self.wait_call:
                f, a, k = self.wait_call.pop(0)
                if f(*a, **k): break


class MenuMode(mode.Mode):

    name = "menu"

    ## Constructor
    ##############

    def __init__(self, state=None):
        super(MenuMode, self).__init__()

        ## General
        ##########

        # Menu state.
        if state is None:
            self.model_state = MenuModelState()
        else:
            self.model_state = state
        self.view_state = MenuViewState(self, self.model_state)

        # Interface root.
        self.root = interface.Root()

        # Quit Icon.
        self.quit_icon = \
        self.root.add(interface.Link,
            xpos = 0.9, ypos = 0.9,
            width = 0.1, height = 0.1,
            text = u"X",
            font_size = 0.04,
            margin = 0.01,
            padding = 0.01,
            halign = "right",
            valign = "top",
            #background = (0, 0, 0, 80),
            #outline = (255, 255, 255, 255),
            callback = self.option_quit,
        )

        # Ship.
        self.ship = \
        self.root.add(Ship,
            graphic = squirtle.SVG(data.image_path(self.model_state.ship_name)),
            height = 0.8,
            fitheight = 0.5,
            aoff = 45,
            popup = 1.2,
            callback = self.option_show_ship_select,
            always_popup = True,
        )

        # Title.
        self.title_text = \
        self.root.add(interface.Text,
            ypos = 0.8, height = 0.2,
            text = u"Intestinvaders!",
            background = (0, 0, 0, 80),
            outline = (255, 255, 255, 255),
            padding = 0.05, font_size = 0.07
        )

        ## Username
        ###########

        # Username.
        self.username = \
        self.root.add(interface.Interface)

        # Username Text.
        self.username_text = \
        self.username.add(interface.Text,
            ypos = 0.6, height = 0.2,
            font_size = 0.04,
            text = u"Enter a username:",
        )

        username = config.username
        if not username:
            try:
                username = os.getlogin()
            except:
                try:
                    username = os.environ['USERNAME']
                except:
                    username = ""
        # Username Entry.
        self.username_entry = \
        self.username.add(interface.Entry,
            ypos = 0.4, height = 0.2,
            default = username,
            font_size = 0.04,
            background = (0, 0, 0, 80),
            outline = (255, 255, 255, 255),
            padding = 0.05,
            callback = self.option_enter_username,
        )


        ## Intro
        ########

        # Intro.
        self.intro = \
        self.root.add(interface.Interface,
            height = 1.0,
        )

        # Intro Text.
        self.intro_text = \
        self.intro.add(interface.Text,
            font_size = 0.025,
            background = (0, 0, 0, 200),
            outline = (255, 255, 255, 255),
            padding = 0.05, margin = 0.05,
        )

        ## Main
        #######

        # Main Menu.
        self.main_menu = \
        self.root.add(interface.Interface,
            height = 0.8,
        )

        # Start Icon.
        self.start_icon = \
        self.main_menu.add(interface.CLink,
            xpos = 0.0, ypos = 0.5,
            width = 0.5, height = 0.5,
            text = u"Start\nGame",
            layout = self.main_menu,
            callback = self.option_season,
        )

        # Single Race Icon.
        self.single_race_icon = \
        self.main_menu.add(interface.CLink,
            xpos = 0.5, ypos = 0.5,
            width = 0.5, height = 0.5,
            text = u"Single\nRace",
            layout = self.main_menu,
            callback = self.option_start_game,
        )

        # Hi-Scores Icon.
        self.hiscores_icon = \
        self.main_menu.add(interface.CLink,
            xpos = 0.5, ypos = 0.0,
            width = 0.5, height = 0.5,
            text = u"Best\nTimes",
            layout = self.main_menu,
            callback = self.option_hiscores,
        )

        # Options Icon.
        self.options_icon = \
        self.main_menu.add(interface.CLink,
            xpos = 0.0, ypos = 0.0,
            width = 0.5, height = 0.5,
            text = u"Game\nOptions",
            layout = self.main_menu,
            callback = self.option_options,
        )

        ## Start
        ########

        self.start = \
        self.root.add(interface.Interface,
            height = 0.8,
        )

        self.start_track_name = \
        self.start.add(interface.Link,
            xpos = 0.4, width = 0.6,
            ypos = 0.8, height = 0.2,
            text = self.model_state.track_name,
            #default = self.model_state.track_name,
            #callback = self.option_enter_start_track,
            font_size = 0.06,
            callback = self.option_start_show_selector,
        )

        self.start_track = \
        self.start.add(interface.Text,
            xpos = 0.0, width = 0.4,
            ypos = 0.8, height = 0.2,
            text = u"Track Name:",
            font_size = 0.04, padding = 0.04,
        )

        #self.start_show_selector = \
        #self.start.add(interface.Link,
            #xpos = 0.0, ypos = 0.75,
            #width = 0.4, height = 0.2,
            #text = u"[Select]",
            #font_size = 0.04, padding = 0.04,
            #callback = self.option_start_show_selector,
            ##background = (0, 0, 0, 80),
            ##outline = (255, 255, 255, 255),
        #)

        self.start_mode = \
        self.start.add(interface.Text,
            xpos = 0.0, width = 0.4,
            ypos = 0.65, height = 0.2,
            text = u"Race Mode:",
            font_size = 0.04, padding = 0.04,
        )

        self.start_mode_tt = \
        self.start.add(interface.Link,
            xpos = 0.4, width = 0.3,
            ypos = 0.65, height = 0.2,
            text = u"Time Trial",
            callback = self.option_start_mode_tt,
            font_size = 0.04, padding = 0.04,
        )

        self.start_mode_gp = \
        self.start.add(interface.Link,
            xpos = 0.7, width = 0.3,
            ypos = 0.65, height = 0.2,
            text = u"Grand Prix",
            callback = self.option_start_mode_gp,
            font_size = 0.04, padding = 0.04,
        )

        self.start_mode_icons = (
            self.start_mode_tt,
            self.start_mode_gp,
        )

        self.start_button = \
        self.start.add(interface.Link,
            xpos = 0.0, ypos = 0.0,
            height = 0.2,
            text = u"Start!",
            callback = self.option_start,
        )

        ## Season
        #########

        self.season = \
        self.root.add(interface.Interface,
            height = 0.8,
        )

        self.season_track_name = \
        self.season.add(interface.Link,
            xpos = 0.4, width = 0.6,
            ypos = 0.8, height = 0.2,
            text = self.model_state.season_track_name,
            #default = self.model_state.track_name,
            #callback = self.option_enter_season_track,
            font_size = 0.06,
            callback = self.option_season_show_selector,
        )

        self.season_track = \
        self.season.add(interface.Text,
            xpos = 0.0, width = 0.4,
            ypos = 0.8, height = 0.2,
            text = u"Season Name:",
            font_size = 0.04, padding = 0.04,
        )

        #self.season_show_selector = \
        #self.season.add(interface.Link,
            #xpos = 0.0, ypos = 0.75,
            #width = 0.4, height = 0.2,
            #text = u"[Select]",
            #font_size = 0.04, padding = 0.04,
            #callback = self.option_season_show_selector,
            ##background = (0, 0, 0, 80),
            ##outline = (255, 255, 255, 255),
        #)

        self.season_button = \
        self.season.add(interface.Link,
            xpos = 0.0, ypos = 0.0,
            height = 0.2,
            text = u"Start!",
            callback = self.option_start_season,
        )

        ## Options
        ##########

        self.options = \
        self.root.add(interface.Tabular,
            height = 0.8, padding = 0.04,
            spacing = 0.04, span = 1,
            background = (0, 0, 0, 80),
            outline = (255, 255, 255, 255),
        )

        self.opt_fs = \
        self.options.add(interface.Link,
            text = u"Fullscreen: Off",
            callback = self.option_fullscreen,
        )
        self._reset_opt("fullscreen", self.opt_fs)

        self.opt_mus = \
        self.options.add(interface.Link,
            text = u"Music: Off",
            callback = self.option_music,
        )
        self._reset_opt("music", self.opt_mus)

        self.opt_sfx = \
        self.options.add(interface.Link,
            text = u"Sound: Off",
            callback = self.option_sound,
        )
        self._reset_opt("sound", self.opt_sfx)

        self.options.add(interface.Link,
            text = u"Switch User",
            callback = self.option_switch,
        )

        ## Hiscores
        ###########

        # Hiscores
        self.hiscores = \
        self.root.add(interface.Interface,
            height = 1.0,
        )

        #self.hiscores_mode_label = \
        #self.hiscores.add(interface.Text,
            #xpos = 0.0, width = 0.4,
            #ypos = 0.88, height = 0.12,
            #text = u"Race Mode:",
            #font_size = 0.032,
        #)

        self.hiscores_season_label = \
        self.hiscores.add(interface.Text,
            xpos = 0.0, width = 0.4,
            ypos = 0.74, height = 0.12,
            text = u"Season Name:",
            font_size = 0.032,
        )

        self.hiscores_track_label = \
        self.hiscores.add(interface.Text,
            xpos = 0.0, width = 0.4,
            ypos = 0.62, height = 0.12,
            text = u"Track Name:",
            font_size = 0.032,
        )

        self.track_name = \
        self.hiscores.add(interface.Link,
            xpos = 0.4, width = 0.6,
            ypos = 0.62, height = 0.12,
            text = self.model_state.hiscore_track_name,
            #default = self.model_state.track_name,
            callback = self.option_start_show_selector,
            font_size = 0.048,
        )

        self.season_name = \
        self.hiscores.add(interface.Link,
            xpos = 0.4, width = 0.6,
            ypos = 0.74, height = 0.12,
            text = self.model_state.hiscore_season_name,
            callback = self.option_season_show_selector,
            font_size = 0.048,
        )

        self.mode_season = \
        self.hiscores.add(interface.Link,
            xpos = 0.0, width = 0.3,
            ypos = 0.90, height = 0.12,
            text = u"Season",
            callback = self.option_mode_season,
            font_size = 0.032, padding = 0.026,
        )

        self.mode_tt = \
        self.hiscores.add(interface.Link,
            xpos = 0.35, width = 0.3,
            ypos = 0.90, height = 0.12,
            text = u"Time Trial",
            callback = self.option_mode_tt,
            font_size = 0.032, padding = 0.026,
        )

        self.mode_gp = \
        self.hiscores.add(interface.Link,
            xpos = 0.7, width = 0.3,
            ypos = 0.90, height = 0.12,
            text = u"Grand Prix",
            callback = self.option_mode_gp,
            font_size = 0.032, padding = 0.026,
        )

        self.hiscore_mode_icons = (self.mode_season, self.mode_tt, self.mode_gp)

        #self.mode_choose = \
        #self.hiscores.add(interface.Link,
            #xpos = 0.0, width = 0.2,
            #ypos = 0.8, height = 0.2,
            #text = u"Grand Prix",
            #callback = self.option_toggle_race_mode,
            #font_size = 0.03, padding = 0.03,
            #background = (0, 0, 0, 80),
            #outline = (255, 255, 255, 255),
        #)

        #self.show_selector = \
        #self.hiscores.add(interface.Link,
            #xpos = 0.8, ypos = 0.8,
            #width = 0.2, height = 0.2,
            #text = u"Select Track",
            #font_size = 0.03, padding = 0.03,
            #callback = self.option_show_selector,
            #background = (0, 0, 0, 80),
            #outline = (255, 255, 255, 255),
        #)

        # Hiscore Table
        self.hiscore_table = \
        self.hiscores.add(interface.Tabular,
            ypos = 0.10,
            height = 0.52, margin = 0.02,
            background = (0, 0, 0, 200),
            outline = (255, 255, 255, 255),
            span = 5, valign = "top",
            spacing = 0.04, padding = 0.04,
        )

        # Hiscore Personal Icon
        self.hiscore_personal_icon = \
        self.hiscores.add(interface.Link,
            xpos = 0.1, width = 0.3,
            ypos = -0.02, height = 0.12,
            text = u"Personal",
            font_size = 0.032, padding = 0.026,
            callback = self.set_personal_hiscores,
        )

        # Hiscore Local Icon
        self.hiscore_local_icon = \
        self.hiscores.add(interface.Link,
            xpos = 0.4, width = 0.3,
            ypos = -0.02, height = 0.12,
            text = u"Local",
            font_size = 0.032, padding = 0.026,
            callback = self.set_local_hiscores,
        )

        # Hiscore Global Icon
        self.hiscore_global_icon = \
        self.hiscores.add(interface.Link,
            xpos = 0.7, width = 0.3,
            ypos = -0.02, height = 0.12,
            text = u"Global",
            font_size = 0.032, padding = 0.026,
            callback = self.set_global_hiscores,
        )

        self.hiscore_icons = (self.hiscore_personal_icon,
                self.hiscore_local_icon, self.hiscore_global_icon)

        # Hiscore Format.
        self.hiscore_format = dict(
            font_size = 0.06,
        )

        # Hiscore Request.
        self.hiscore_request = None
        self.cloud_tex = data.load_image('clouds.png').texture
        self.cloud_pos = v(0,0)
        self.cloud_vel = v(0,0)
        self.cloud_acc = v(0,0)

        ## Selector
        ###########

        self.selector = \
        self.root.add(interface.Interface,
            height = 0.8,
        )

        self.selector_tabel = \
        self.selector.add(Tabel,
            height = 0.8,
            font_size = 0.025, hspacing = 0.08,
            spacing = 0.01, span = 3,
            padding = 0.02, margin = 0.02,
            background = (0, 0, 0, 200),
            outline = (255, 255, 255, 255),
            callback = self.option_selector_select,
            texts = presidents.order + ["Random"],
        )

        self.selector_tabel_season = \
        self.selector.add(Tabel,
            height = 0.8,
            font_size = 0.04, hspacing = 0.08,
            spacing = 0.01, span = 1,
            padding = 0.04, margin = 0.04,
            background = (0, 0, 0, 200),
            outline = (255, 255, 255, 255),
            callback = self.option_selector_select,
            texts = (),
            show = False,
        )

        ## Season Selector
        ##################

        self.season_selector = \
        self.root.add(interface.Interface,
            height = 0.8,
        )

        self.season_selector_tabel = \
        self.season_selector.add(Tabel,
            height = 0.8,
            font_size = 0.04, hspacing = 0.08,
            spacing = 0.01, span = 2,
            padding = 0.04, margin = 0.04,
            background = (0, 0, 0, 200),
            outline = (255, 255, 255, 255),
            callback = self.option_season_selector_select,
            texts = presidents.seasonorder,
        )

        ## Ship Selector
        ################

        self.ship_selector = \
        self.root.add(interface.Interface,
            height = 0.8,
        )

        self.ship_selector_table = \
        self.ship_selector.add(interface.Tabular,
            height = 1.0, span = 5,
            padding = 0.08, margin = 0.08,
            spacing = 0.08,
            background = (0, 0, 0, 80),
            outline = (255, 255, 255, 255),
        )

        self.ships = [
            "argentineship.svg",
            "canadaship.svg",
            "confedship.svg",
            "frenchship.svg",
            "germanship.svg",
            "greeceship.svg",
            "irishship.svg",
            "italianship.svg",
            "jamaicaship.svg",
            "japanship.svg",
            "lgbtship.svg",
            "southafricaship.svg",
            "ukship.svg",
            "usaship.svg",
            "ussrship.svg",
        ]

        for imgname in self.ships:
            g = squirtle.SVG(data.image_path(imgname))
            self.ship_selector_table.add(interface.Image,
                graphic = g, fitwidth = 0.10, aoff = 90,
                callback = (self.option_ship_select, imgname),
                popup = 1.2
            )

    ## Mode methods
    ###############

    def connect(self, control):
        super(MenuMode, self).connect(control)
        self.control.music.change_song('show.ogg')
        self.window.set_mouse_visible(True)
        self.on_resize(*self.window.get_size())
        self.root.offset = self.view_offset
        self.root.size = self.view_size
        self.root.window = self.window
        self.username.window = None
        if self.model_state.show_username:
            self.username.window = self.window


    def disconnect(self):
        self.root.window = None
        super(MenuMode, self).disconnect()

    def update_view(self):
        self.view_state.tick()
        if self.model_state.show_main or (self.model_state.show_start and not self.model_state.show_track_choose) or (self.model_state.show_season and not self.model_state.show_season_choose):
            self.ship._ign_event = False
        else:
            self.ship._mouse_in_interface = False
            self.ship._ign_event = True
        self.title_text.yoff = _ease(self.view_state.title_offset) * \
                self.title_text.box_size[1]
        self.title_text.show = self.view_state.show_title
        self.username.yoff = _ease(self.view_state.username_offset) * \
                -self.username.layout_size[1]
        self.username.show = self.view_state.show_username
        self.intro.yoff = _ease(self.view_state.intro_offset) * \
                -self.intro.real_height
        self.intro.show = self.view_state.show_intro
        self.main_menu.yoff = _ease(self.view_state.main_offset) * \
                -self.main_menu.box_size[1]
        self.main_menu.show = self.view_state.show_main
        self.start.xoff = _pop(self.view_state.start_offset) * \
                self.start.real_width
        self.start.show = self.view_state.show_start
        self.season.xoff = _pop(self.view_state.season_offset) * \
                -self.season.real_width
        self.season.show = self.view_state.show_season
        self.options.xoff = _ease(self.view_state.options_offset) * \
                -self.options.real_width
        self.options.show = self.view_state.show_options
        self.hiscores.xoff = _ease(self.view_state.hiscores_offset) * \
                self.hiscores.real_width
        self.hiscores.show = self.view_state.show_hiscores
        self.selector.yoff = _pop(self.view_state.track_choose_offset) * \
                -self.selector.real_height
        self.selector.show = self.view_state.show_track_choose
        self.season_selector.yoff = _pop(self.view_state.season_choose_offset) * \
                -self.season_selector.real_height
        self.season_selector.show = self.view_state.show_season_choose
        self.ship_selector.yoff = _pop(self.view_state.ship_choose_offset) * \
                -self.ship_selector.real_height
        self.ship_selector.show = self.view_state.show_ship_choose

        # HS Season
        offset = _ease(self.view_state.hsseason_offset)
        self.hiscores_track_label.yoff = offset * \
                self.hiscores_track_label.real_height
        self.track_name.yoff = offset * \
                self.track_name.real_height
        self.hiscore_table.yoff = offset * self.track_name.real_height
        self.hiscores_season_label.show = self.view_state.show_hsseason
        self.season_name.show = self.view_state.show_hsseason
        if self.view_state.show_hsseason:
            alpha = int(self.view_state.hsseason_fade * 255)
            self.hiscores_season_label.color = \
                    self.hiscores_season_label.color[:3] + (alpha,)
            self.season_name.color = self.season_name.color[:3] + (alpha,)

        self.ship.popup = 1.0 + 0.2 * _ease(self.view_state.ship_popup)
        if self.model_state.show_track_choose:
            self.start_button.leave()
            self.start_button._mouse_in_interface = False

        if self.model_state.show_hiscores and \
                self.model_state.hiscore_mode == u"Season":
            self.selector_tabel_season.texts = \
                    presidents.seasons[self.model_state.hiscore_season_name]
            self.selector_tabel_season.show = True
            self.selector_tabel.show = False
        else:
            self.selector_tabel_season.show = False
            self.selector_tabel.show = True

        for i in self.hiscore_icons:
            i.background = None
            i.outline = None
        i = getattr(self, "hiscore_%s_icon" % self.model_state.hiscore_scope)
        i.background = (0, 0, 0, 80)
        i.outline = (255, 255, 255, 255)

        for i in self.hiscore_mode_icons:
            if self.model_state.hiscore_mode != i.text:
                i.background = None
                i.outline = None
            else:
                i.background = (0, 0, 0, 80)
                i.outline = (255,255,255,255)

        for i in self.start_mode_icons:
            if self.model_state.race_mode != i.text:
                i.background = None
                i.outline = None
            else:
                i.background = (0, 0, 0, 80)
                i.outline = (255,255,255,255)

        for i, c in enumerate(self.main_menu._components):
            t = 1 - self.view_state.main_offset
            if i % 2: t = -t
            c.aoff = _wibble(t, 4) * 15.0
        
        self.cloud_pos += self.cloud_vel
        self.cloud_vel += self.cloud_acc
        if random.random() < 0.01: self.cloud_acc += v(random.gauss(0,0.5), random.gauss(0,0.5))
        self.cloud_vel *= .95

        if not hasattr(self, "__cheatycheatycheat__"):
            self.__cheatycheatycheat__ = None
            self.reload_hiscores()

    ## Event handlers
    #################

    def on_key_press(self, sym, mods):
        from pyglet.window import key
        from pyglet.event import EVENT_HANDLED
        from pyglet.event import EVENT_UNHANDLED
        if self.model_state.show_main:
            if sym == key.ESCAPE:
                self.option_quit()
            else:
                return EVENT_UNHANDLED
        elif self.model_state.show_intro:
            self.model_state.set_main()
        elif sym == key.ESCAPE:
            if self.model_state.show_username:
                pass
            elif self.model_state.show_start and \
                       (self.model_state.show_track_choose or
                        self.model_state.show_ship_choose):
                    self.model_state.set_start()
            elif self.model_state.show_season and \
                    self.model_state.show_season_choose:
                    self.model_state.set_season()
            elif self.model_state.show_hiscores and \
                    (self.model_state.show_track_choose or
                     self.model_state.show_season_choose):
                    self.model_state.set_hiscores()
            elif not self.model_state.exiting:
                self.model_state.set_main()
            else:
                return EVENT_UNHANDLED
        else:
            return EVENT_UNHANDLED
        return EVENT_HANDLED

    def on_rpc_response(self, request, response):
        if request == self.hiscore_request:
            self._change_hiscore_table(response)
            self.hiscore_request = None
        else:
            return pyglet.event.EVENT_UNHANDLED
        return pyglet.event.EVENT_HANDLED

    def on_rpc_error(self, request, error):
        if request == self.hiscore_request:
            self.hiscore_request = None
            self.hiscore_table.add(interface.Text, text=u"Error!",
                    **self.hiscore_format)
        else:
            return pyglet.event.EVENT_UNHANDLED
        return pyglet.event.EVENT_HANDLED

    ## Hi-score management
    ######################

    _modes = {u"Time Trial": "TT", u"Grand Prix": "GP"}

    def _get_stuff(self, track=None):
        mode = self.model_state.hiscore_mode
        mode = self._modes.get(mode, mode)
        track = track or self.model_state.hiscore_track_name
        if mode == u"Season":
            mode = self.model_state.hiscore_season_name
        return mode, track

    def set_personal_hiscores(self, track=None):
        mode, track = self._get_stuff()
        self.model_state.hiscore_scope = "personal"
        scores = hiscores.get_personal_scores(mode, track)
        self._change_hiscore_table(scores)

    def set_local_hiscores(self, track=None):
        mode, track = self._get_stuff()
        self.model_state.hiscore_scope = "local"
        scores = hiscores.get_local_scores(mode, track)
        self._change_hiscore_table(scores)

    def set_global_hiscores(self, track=None):
        mode, track = self._get_stuff()
        self.model_state.hiscore_scope = "global"
        self.hiscore_request = self.control.rpc.get_hiscores(mode, track)
        self.hiscore_request.listen(self)
        self.hiscore_table.clear()
        self.hiscore_table.add(interface.Text, text=u"Loading...",
                               **self.hiscore_format)

    def reload_hiscores(self):
        if self.model_state.hiscore_scope == "personal":
            self.set_personal_hiscores()
        elif self.model_state.hiscore_scope == "local":
            self.set_local_hiscores()
        elif self.model_state.hiscore_scope == "global":
            self.set_global_hiscores()

    def _change_hiscore_table(self, scores):
        self.hiscore_table.clear()
        if not scores:
            self.hiscore_table.add(interface.Text, text=u"No times.",
                    **self.hiscore_format)
            return
        for mode, user, race, score, date, ghost in scores:
            score, date = formatscore(score), formatdate(date)
            for text in (user, score, date):
                self.hiscore_table.add(interface.Text, text=text,
                                       **self.hiscore_format)
            self.hiscore_table.add(interface.Link, text=u"[Race]",
                    callback = (self.option_start_ghost_race,
                        mode, race, ghost),
                    **self.hiscore_format)
            self.hiscore_table.add(interface.Link, text=u"[Watch]",
                    callback = (self.option_start_watch_race,
                        mode, race, ghost),
                    **self.hiscore_format)

    ## Option callbacks
    ###################

    tracks = presidents.order

    def option_next_track(self):
        try:
            idx = self.tracks.index(self.model_state.track_name)
        except ValueError:
            idx = -1
        idx = (idx + 1) % len(self.tracks)
        self.track_name.default = self.tracks[idx]
        self.model_state.track_name = self.tracks[idx]
        self.reload_hiscores()

    def option_previous_track(self):
        try:
            idx = self.tracks.index(self.model_state.track_name)
        except ValueError:
            idx = 0
        idx = (idx - 1) % len(self.tracks)
        self.track_name.default = self.tracks[idx]
        self.model_state.track_name = self.tracks[idx]
        self.reload_hiscores()

    def option_enter_username(self, text):
        if re.match(ur"^[-_\.a-zA-Z0-9]+$", text):
            already_have = config.username is not None
            config.save_option("username", text)
            self.username_entry.default = text
            if not already_have:
                self.option_show_intro()
            else:
                self.model_state.set_main()
            self.username.window = None

    def option_toggle_start_race_mode(self):
        modes = [u"Grand Prix", u"Time Trial"]
        t = self.start_mode_choose.text
        self.start_mode_choose.text = modes[(modes.index(t) + 1) % 2]
        self.mode_choose.text = modes[(modes.index(t) + 1) % 2]
        self.model_state.race_mode = modes[(modes.index(t) + 1) % 2]
    def option_toggle_race_mode(self):
        modes = [u"Grand Prix", u"Time Trial"]
        t = self.mode_choose.text
        self.start_mode_choose.text = modes[(modes.index(t) + 1) % 2]
        self.mode_choose.text = modes[(modes.index(t) + 1) % 2]
        self.model_state.race_mode = modes[(modes.index(t) + 1) % 2]
        self.reload_hiscores()

    def option_start_mode_tt(self):
        self.model_state.race_mode = u"Time Trial"
    def option_start_mode_gp(self):
        self.model_state.race_mode = u"Grand Prix"
    def option_mode_season(self):
        self.model_state.hiscore_mode = u"Season"
        self.reload_hiscores()
        self.option_enter_track(presidents.seasons[self.model_state.hiscore_season_name][0])
    def option_mode_tt(self):
        self.model_state.hiscore_mode = u"Time Trial"
        self.reload_hiscores()
        self.model_state.show_season_choose = False
    def option_mode_gp(self):
        self.model_state.hiscore_mode = u"Grand Prix"
        self.reload_hiscores()
        self.model_state.show_season_choose = False

    def option_season_show_selector(self):
        if self.model_state.show_season:
            self.model_state.toggle_season_choose()
        elif self.model_state.show_hiscores:
            if self.model_state.hiscore_mode == u"Season":
                self.model_state.toggle_season_choose()
    def option_start_show_selector(self):
        if self.model_state.show_start:
            self.model_state.toggle_track_choose()
        elif self.model_state.show_hiscores:
            self.model_state.toggle_track_choose()
    def option_show_selector(self):
        self.model_state.toggle_track_choose()

    def option_enter_track(self, text):
        if text.strip():
            self.track_name.text = presidents.resolve(text.strip())
            self.start_track_name.text = presidents.resolve(text.strip())
            self.model_state.hiscore_track_name = presidents.resolve(text.strip())
            self.reload_hiscores()
    def option_enter_season(self, text):
        if text.strip():
            self.season_name.text = presidents.resolve(text.strip())
            self.model_state.hiscore_season_name = presidents.seasonresolve(text.strip())
            self.option_enter_track(presidents.seasons[self.model_state.hiscore_season_name][0])
            self.reload_hiscores()
    def option_enter_season_track(self, text, start=True):
        if text.strip():
            text = text.strip()
            self.season_track_name.text = presidents.seasonresolve(text)
            self.model_state.season_track_name = presidents.seasonresolve(text)
    def option_enter_start_track(self, text, start=True):
        if text.strip():
            self.track_name.text = presidents.resolve(text.strip())
            self.start_track_name.text = presidents.resolve(text.strip())
            self.model_state.track_name = presidents.resolve(text.strip())
    def option_season_selector_select(self, text):
        self.model_state.toggle_season_choose()
        if self.model_state.show_season:
            self.option_enter_season_track(text)
        elif self.model_state.show_hiscores:
            self.option_enter_season(text)
    def option_selector_select(self, text):
        self.model_state.toggle_track_choose()
        if text == 'Random':
            text = random.choice(presidents.order)
        if self.model_state.show_start:
            self.option_enter_start_track(text)
        elif self.model_state.show_hiscores:
            self.option_enter_track(text)

    def option_show_ship_select(self):
        self.model_state.set_ship_choose()
    def option_ship_select(self, imgname):
        config.save_option("ship", imgname)
        self.model_state.set_back()
        self.model_state.ship_name = imgname
        self.ship.graphic = squirtle.SVG(data.image_path(imgname))

    def option_start(self):
        mode = ["TT", "GP"][self.model_state.race_mode == "Grand Prix"]
        ms = self.model_state.copy()
        self.model_state.set_starting()
        self.view_state.wait_animation(self.control.switch_handler,
                "race", racename=self.start_track_name.text,
                shipimg=self.model_state.ship_name,
                race_mode=mode, return_to=ms)
        self.control.music.fadeout(1)
    def option_start_ghost_race(self, mode, race, ghost):
        ms = self.model_state.copy()
        self.model_state.set_starting()
        self.view_state.wait_animation(self.control.switch_handler,
                "race", racename=race,
                shipimg=self.model_state.ship_name,
                ghost=ghost, race_mode=mode,
                return_to=ms)
        self.control.music.fadeout(1)
        
    def option_start_watch_race(self, mode, race, ghost):
        ms = self.model_state.copy()
        self.model_state.set_starting()
        self.view_state.wait_animation(self.control.switch_handler,
                "race", racename=race, ghost=ghost, race_mode=mode,
                interaction="watch", return_to=ms)
        self.control.music.fadeout(1)
    
    def option_start_season(self):
        ms = self.model_state.copy()
        self.model_state.set_starting()
        s = season.Season(self.model_state.season_track_name)
        self.view_state.wait_animation(self.control.switch_handler,
                "race", racename=s.get_next_race(), ghost=None, race_mode=s.name,
                shipimg=self.model_state.ship_name, season=s, return_to=ms)
        self.control.music.fadeout(1)


    def option_show_intro(self):
        self.intro_text.text = intro_text# % config.username
        self.model_state.set_intro()
        it = self.intro_text
        tl = it._text_layout
    def option_season(self):
        self.model_state.set_season()
    def option_start_game(self):
        self.model_state.set_start()
    def option_hiscores(self):
        self.reload_hiscores()
        self.model_state.set_hiscores()
    def option_switch(self):
        self.model_state.set_username()
        self.username.window = self.window
    def option_options(self):
        self.model_state.set_options()
    def option_quit(self):
        if self.model_state.show_main:
            self.model_state.set_exit()
            self.view_state.wait_animation(self.window.close)
        else:
            self.model_state.set_main()

    def _toggle_opt(self, name, interface):
        x, t = interface.text.split(":")
        new_val = (t.strip() == "Off")
        config.save_option(name, new_val)
        self._reset_opt(name, interface)
    def _reset_opt(self, name, interface):
        x, t = interface.text.split(":")
        new_t = ["Off", "On"][config.__dict__[name]]
        interface.text = x + ": " + new_t
    def option_fullscreen(self):
        self._toggle_opt("fullscreen", self.opt_fs)
        #self.window.set_fullscreen(config.fullscreen)
        #def reset(node):
        #    node._clean = False
        #    node.window = self.window
        #    for x in node._cached:
        #        delattr(node, x)
        #    for c in node._components:
        #        reset(c)
        #reset(self.root)
    def option_sound(self):
        self._toggle_opt("sound", self.opt_sfx)
    def option_music(self):
        self._toggle_opt("music", self.opt_mus)
        if not config.music and self.control.music.player.source is not None:
            self.control.music.requeue(None)
        elif config.music:
            self.control.music.change_song('show.ogg', 1.0, 1.0)

    ## Rendering
    ############

    #def on_resize(self, width, height):
        #glViewport(0, 0, width, height)
        #glMatrixMode(GL_PROJECTION)
        #glLoadIdentity()
        #glOrtho(0, width, 0, height, -1, 1)
        #glMatrixMode(GL_MODELVIEW)
        #glLoadIdentity()
        #return pyglet.event.EVENT_HANDLED

    def on_resize(self, width, height):
        subwidth, subheight = width, height
        if WORLD_WIDTH * height > WORLD_HEIGHT * width:
            subheight = width * WORLD_HEIGHT // WORLD_WIDTH
        elif WORLD_WIDTH * height < WORLD_HEIGHT * width:
            subwidth = height * WORLD_WIDTH // WORLD_HEIGHT
        xoff = (width - subwidth) // 2
        yoff = (height - subheight) // 2
        # Remember these values for later.
        self.view_offset = (xoff, yoff)
        self.view_size = (subwidth, subheight)
        # Perform the normal ortho projection.
        glViewport(xoff, yoff, subwidth, subheight)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(0, subwidth, 0, subheight, -1, 1)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        return pyglet.event.EVENT_HANDLED

    def on_draw(self):
        glClearColor(0.0, 0.0, 0.0, 1.0)
        glEnable(GL_LINE_SMOOTH)
        self.update_view()
        if not self.window:
            return
        self.window.clear()
        fx.draw_background((self.window.width, self.window.height), self.cloud_pos, self.cloud_tex)
        self.root.draw()
        if self.view_state.fade_out > 0.0:
            w, h = self.window.get_size()
            f = int(self.view_state.fade_out * 255)
            pyglet.graphics.draw(4, GL_QUADS,
                                 ('v2f', [0, 0, w, 0, w, h, 0, h]),
                                 ('c4B', [0, 0, 0, f] * 4))
