import pyglet

import data

import config
from common import *
from constants import *

class FadingPlayer(pyglet.media.Player):

    def __init__(self):
        super(FadingPlayer, self).__init__()
        self.fading = False
        self.fade_time = 0.0
        self.fade_grace = 0.0
        self.fade_rate = 0.0
        self.volume = 0.0

    def start_fade(self, target, time, grace):
        self.fading = True
        self.fade_time = time
        self.fade_grace = grace
        self.fade_rate = (min(1, max(0, target)) - self.volume) / time

    def stop_fade(self, dt):
        self.fade_grace -= dt
        if self.fade_grace <= 0.0:
            self.fading = False
            self.fade_time = 0.0
            self.fade_grace = 0.0
            self.fade_rate = 0.0

    def dispatch_events(self, dt=None):
        if dt is not None:
            if self.fade_time > 0.0:
                self.fade_time = max(0.0, self.fade_time - dt)
                self.volume = min(1.0, max(0.0,
                    self.volume + self.fade_rate * dt))
            if self.fade_time == 0.0:
                self.stop_fade(dt)
        super(FadingPlayer, self).dispatch_events(dt)


class MusicManager(object):

    def __init__(self):
        if not pyglet.media.have_avbin: return
        self.queue = []
        self.volume = 1.0
        self.player = FadingPlayer()
        self.fade_grace = 0.5
        self.player.eos_action = "loop"
        self.song = None

    def fadein(self, fadein=None):
        if not pyglet.media.have_avbin: return
        if fadein is not None:
            self.player.start_fade(self.volume, fadein, self.fade_grace)
        else:
            self.player.volume = self.volume

    def fadeout(self, fadeout=None):
        if not pyglet.media.have_avbin: return
        if fadeout is not None:
            self.player.start_fade(0.0, fadeout, self.fade_grace)
        else:
            self.player.volume = 0.0

    def requeue(self, song):
        if not pyglet.media.have_avbin: return
        self.player.next()
        if config.music:
            source = data.load_song(song)
            self.player.queue(source)
            self.player.play()
            self.song = song

    def change_song(self, song, fadeout=None, fadein=None):
        if not pyglet.media.have_avbin: return
        if self.player.source is not None:
            self.order("fadeout", fadeout)
        self.order("requeue", song)
        self.order("fadein", fadein)

    def order(self, name, *args, **kwds):
        if not pyglet.media.have_avbin: return
        func = getattr(self, name)
        self.queue.insert(0, (func, args, kwds))

    def update(self, dt=None):
        if not pyglet.media.have_avbin: return
        if self.player.fading:
            pass
        elif self.queue:
            func, args, kwds = self.queue.pop()
            func(*args, **kwds)
            self.update()
        if dt is not None:
            self.player.dispatch_events(dt)


if __name__ == "__main__":
    w = pyglet.window.Window()
    m = MusicManager()
    m.change_song("Radio Martini.ogg")
    pyglet.clock.schedule_interval(m.update, 0.02)
    @w.event
    def on_key_press(sym, mods):
        from pyglet.window import key
        import random, os
        if sym == key.SPACE:
            s = random.choice(os.listdir("data/music"))
            m.change_song(s, 2.0, 2.0)
            print s
    @w.event
    def on_draw():
        w.clear()
    pyglet.app.run()
