from pyglet.gl import *

import random

class ParticleSystem(object):
    def __init__(self, tex, max_n):
        self.tex = tex
        self.max_n = max_n
        self.vdata = (GLfloat * (2*max_n))()
        self.cdata = (GLubyte * (4*max_n))()
        self.vels = [0,0] * max_n
        self.usable = range(max_n)
        
    def tick(self):
        for i in xrange(self.max_n):
            ci = i * 4
            vi = i * 2
            if not self.cdata[ci+3]: continue
            self.cdata[ci+3] -= 5
            if self.cdata[ci+3] <= 5:
                self.cdata[ci+3] = 0
                self.usable.append(i)
                continue
            self.vdata[vi] += self.vels[vi]
            self.vdata[vi+1] += self.vels[vi+1]
            
                
    def add_particle(self, pos, vel, color=(200, 160, 150, 150)):
        if self.usable:
            i = self.usable.pop()
        else:
            i = random.randrange(self.max_n)
            
        self.vdata[2*i:2*i+2] = pos
        self.vels[2*i:2*i+2] = vel
        self.cdata[4*i:4*i+4] = color
    
    def draw(self):
        glEnable(GL_POINT_SPRITE)
        glEnable(GL_TEXTURE_2D)
        glTexEnvf(GL_POINT_SPRITE, GL_COORD_REPLACE, GL_TRUE)
        glBindTexture(GL_TEXTURE_2D, self.tex.id)
        glEnableClientState(GL_VERTEX_ARRAY)
        glEnableClientState(GL_COLOR_ARRAY)
        glDisableClientState(GL_TEXTURE_COORD_ARRAY)
        glVertexPointer(2, GL_FLOAT, 0, self.vdata)
        glColorPointer(4, GL_UNSIGNED_BYTE, 0, self.cdata)
        glDrawArrays(GL_POINTS, 0, self.max_n)
        glDisableClientState(GL_COLOR_ARRAY)
        
        glDisable(GL_POINT_SPRITE)
        glDisable(GL_TEXTURE_2D)