from __future__ import division
from pyglet.gl import *
import math
from constants import *
from common import *
import bezier
from vector import v

def bezier_patch(bez, width, **kwargs):
    return bezier_wall(bez, width, -width, **kwargs)
    
def bezier_quad(b1, b2, sections=40, wobble_depth=20, wobble_freq=4):
    vdata = []
    tdata = []
    idxs = []
    wobble_dir = (b1.pos(0) - b2.pos(0)).normalised()
    
    def lerp(x0, x1, a):
        return x0 + a * (x1 - x0)
    for i in xrange(sections + 1):
        a = i / sections
        b = bezier.Bezier(lerp(b1.p0, b2.p0, a), lerp(b1.p1, b2.p1, a), lerp(b1.p2, b2.p2, a), lerp(b1.p3, b2.p3, a))
        for j in xrange(sections + 1):
            t = j / sections
            tdata.extend((a, 2 * t * wobble_freq))
            wobble = abs(math.sin(2 * math.pi * t * wobble_freq))
            vdata.extend(b.pos(t) + wobble * -2 * (a - .5) * wobble_depth *wobble_dir)
            if i < sections and j < sections:
                idxs.extend((i * (sections + 1) + j, i * (sections + 1) + j + 1, (i + 1) * (sections + 1) + j + 1, (i + 1) * (sections + 1) + j))
    idxs = (GLint * len(idxs))(*idxs)
    vdata = (GLfloat * len(vdata))(*vdata)
    tdata = (GLfloat * len(tdata))(*tdata)
    return idxs, vdata, tdata

def bezier_wall(bez, offsets, t0, t1, batch=None, group=None, sections=SPLINE_RESOLUTION):
    vdata = []
    tdata = []
    for i in xrange(sections + 1):
        t = t0 + i / sections * (t1 - t0)
        c = bez.pos(t)
        n = bez.normal(t)
        for off1, off2 in pairs(offsets):
            vdata.extend(c + off1 * n)
            vdata.extend(c + off2 * n)
            tdata.extend((0, t))
            tdata.extend((1, t))
    nos = len(offsets) - 1
    idxs = []
    for n in xrange(nos):
        idxs.append(2*n)
        for i in xrange(sections + 1):
            idxs.append(2*i*nos + 2*n)
            idxs.append(2*i*nos + 2*n + 1)
        idxs.append(idxs[-1])
        
    idxs = (GLint * len(idxs))(*idxs)
    vdata = (GLfloat * len(vdata))(*vdata)
    tdata = (GLfloat * len(tdata))(*tdata)
    return idxs, vdata, tdata