from __future__ import division

import mode
import config
import interface

from pyglet.gl import *
from model.entity import Entity, TrackEntity
from model.player import Player
from model.race import Race
from bezier import Bezier
import patch
import squirtle
import particle
import fx
import bezier
import data
import math
import weakref
import random
from vector import v, zero
from constants import *
from common import *
import hiscores
import time
import sound
import season
import presidents



class RaceMode(mode.Mode):
    name = 'race'
    
    def __init__(self, *args, **kwargs):
        if "racename" in kwargs:
            name = kwargs.pop("racename")
        elif config.race_name is not None:
            name = config.race_name
        else:
            assert False
        name = presidents.resolve(name)
        
        self.return_to = kwargs.pop("return_to", None)
        
        self.season = kwargs.pop("season", None)
        
        self.race_mode = kwargs.pop("race_mode", "GP")
        self.interaction_mode = kwargs.pop("interaction", "play")
        
        if self.season is None and self.race_mode not in ("GP", "TT"):
            # doing a season replay, at least I hope; need a season object for the race to
            # generate the list of racers from
            self.season = season.Season(self.race_mode, single=True)
        
        ghost = kwargs.pop("ghost", None)
        if ghost is not None:
            ghost = hiscores.decode_ghost(ghost)
        elif self.race_mode == "TT":
            hss = hiscores.get_personal_scores(self.race_mode, name)
            if hss:
                ghost = hiscores.decode_ghost(hss[0][5])
        
        if self.interaction_mode == "play":
            self.race = Race(name, self.race_mode, self.interaction_mode, None, season=self.season)
            self.ghost_race = None
            if ghost is not None:
                self.ghost_race = Race(name, self.race_mode, self.interaction_mode, ghost)
                self.ghost_race.player.svg_name = ghostify_svg_name(ghost.get('image', 'ukship.svg'))
        elif self.interaction_mode == "watch" and ghost is not None:
            self.race = Race(name, self.race_mode, self.interaction_mode, ghost, season=self.season)
            self.ghost_race = None
        else:
            assert False

        p = self.race.player
        if p.is_ghost:
            p.svg_name = p.ghostdata.get('image', 'ukship.svg')
        else:
            p.svg_name = kwargs.pop("shipimg", "usaship.svg")
            self.race.new_ghostdata['image'] = p.svg_name
            self.race.new_ghostdata['pilot'] = config.username

        if DEBUG:
            self.fps = pyglet.clock.ClockDisplay()    
        super(RaceMode, self).__init__(*args, **kwargs)
        self.race.push_handlers(self)
        self.stopping = False

        self.batch = pyglet.graphics.Batch()
        self.dummy_batch = pyglet.graphics.Batch()
        self.track_tex = data.load_image('track.png').texture
        self.svgs = {}
        self.cloud_tex = data.load_image('clouds.png').texture
        self.obstacle_tex = data.load_image('blob.png').texture
        
        self.particles = particle.ParticleSystem(data.load_image('puff.png').texture, 512)
        
        b_pts = [v(0, self.race.track.width + 50), v(-50, self.race.track.width + 150), v(-900, 100), v(-900, 0)]
        b1 = bezier.Bezier(*b_pts)
        b2 = bezier.Bezier(*[v(x[0], -x[1]) for x in b_pts])
        self.duodena = []
        self.duodenum_tex = data.load_image("duodenum.png").texture
        self.duodena.append(patch.bezier_quad(b1, b2))
        endp = self.race.track.pos(len(self.race.track.segs))
        endt = self.race.track.tangent(len(self.race.track.segs))
        angle = endt.angle + 180
        
        b1 = bezier.Bezier(*[endp + x.rotated(angle) for x in b_pts])
        b2 = bezier.Bezier(*[endp + v(x[0], -x[1]).rotated(angle) for x in b_pts])
        self.duodena.append(patch.bezier_quad(b1, b2))
        self.walls = []
        for n, b in enumerate(self.race.track.segs):
            for i in xrange(WALL_SECTIONS):
                t0 = i / WALL_SECTIONS
                t1 = (i+1) / WALL_SECTIONS
                self.walls.append(patch.bezier_wall(b, [self.race.track.width + 50, self.race.track.width-20, -self.race.track.width + 20, -self.race.track.width-50], t0, t1))
        self.wall_vxs = [w[1][:] for w in self.walls]
        self.correct_walls = [w[1][:] for w in self.walls]
        self.wall_vels = [[0 for x in w[1]] for w in self.walls]
        self.dead_walls = [False for w in self.walls]
        self.minimap_data = None
        
        # Interface root.
        self.root = interface.Root()

        # Timer.
        self.timer = \
        self.root.add(interface.Timer,
            ypos = 0.8, height = 0.2,
            halign = "center",
            font_name = "Anamorphic",
            font_size = 0.04, padding = 0.02,
        )

        self.countdown = \
        self.root.add(interface.Text,
            height = 1.0,
            text = u"4",
            color = (255, 255, 255, 0),
            halign = "center",
            font_name = "Anamorphic",
            font_size = 0.4
        )
        self.countdown_ticks = 200
        self.started = False
        
        self.position = \
        self.root.add(interface.Position,
            ypos = 0.79, xpos=0.05,
            width = 0.3,
            height = 0.2,
            text = u"",
            color = (255, 255, 255, 127),
            halign = "left",
            font_name = "Anamorphic",
            font_size = 0.06
        )
        if self.race_mode == 'TT':
            self.position.show = False
        
        self.summary_pos = \
        self.root.add(interface.Text,
            height = 0.8,
            width = 0.7,
            xpos = 0.15,
            ypos = 0.1,
            text = "",
            color = (255, 255, 255, 0),
            halign = "left",
            font_name = "Anamorphic",
            font_size = 0.04
        )
        self.summary_name = \
        self.root.add(interface.Text,
            height = 0.8,
            width = 0.7,
            xpos = 0.3,
            ypos = 0.1,
            text = "",
            color = (255, 255, 255, 0),
            halign = "left",
            font_name = "Anamorphic",
            font_size = 0.04
        )
        
        if self.season and not self.season.has_next_race():
            txt = "Final Results"
        else:
            txt = "Season Standings"
        self.season_head = \
        self.root.add(interface.Text,
            height = 0.16,
            ypos = 0.8,
            text = txt,
            color = (255, 255, 255, 255),
            halign = "center",
            font_name = "Anamorphic",
            font_size = 0.08
        )
        self.season_head.show = False
        
        self.summary_time = \
        self.root.add(interface.Text,
            height = 0.8,
            width = 0.7,
            xpos = 0.65,
            ypos = 0.1,
            text = "",
            color = (255, 255, 255, 0),
            halign = "left",
            font_name = "Anamorphic",
            font_size = 0.04
        )
        self.fade_alpha = 0
        self.finished = False
        self.showing_season = False
        
        self.track_name = \
        self.root.add(interface.Text,
            height = 0.1,
            ypos = 0.18,
            text = name,
            color = (255, 255, 255, 255),
            halign = "center",
            font_name = "Anamorphic",
            font_size = 0.06
        )
        
        self.track_fact = \
        self.root.add(interface.Text,
            height = 0.08,
            ypos = 0.11,
            text = u'\u2026' + presidents.facts[name],
            color = (255, 255, 255, 255),
            halign = "center",
            font_name = "Anamorphic",
            font_size = 0.02
        )
        
    def on_explosion(self, pos, magnitude):
        for a in xrange(0, 360, int(1000/magnitude)):
            p = pos + v(random.gauss(0,10), random.gauss(0, 10))
            vm = random.gauss(1, 1)
            vel = v(vm * magnitude * 0.01, 0).rotated(a)
            if vm < 1:
                c = max(0, min(255, int(vm * 192)))
                self.particles.add_particle(p, vel, (min(c+128, 255), c, 0, random.randrange(128, 256)))
            else:
                self.particles.add_particle(p, vel)

    def on_wall_bump(self, bump_pos, theta, vel, ent):
        if vel < 1: return
        vel = vel ** .5
        blob_dist = 300 ** 2
        bvec = self.race.track.tangent(theta).perpendicular()
        bpos = self.race.track.pos(theta)
        for idx in xrange(max(int(theta - 2), 0), min(int(theta + 2), len(self.walls))):
            if abs((idx + .5) / WALL_SECTIONS - theta) > 2: continue
            wall_pos = self.walls[idx][1]
            for i in xrange(len(wall_pos) // 2):
                p = wall_pos[2*i:2*i+2]
                d = (((p[0] - bump_pos[0]) ** 2 + (p[1] - bump_pos[1]) ** 2) / blob_dist)
                if d > 1: continue
                blob_amount = 2 * vel * (1 + 2 * d) * (1-d) ** 2
                self.dead_walls[idx] = False
                
                if (p[0] - bpos[0]) * bvec[0] + (p[1] - bpos[1]) * bvec[1] < 0: blob_amount *= -1

                self.wall_vels[idx][2*i] += blob_amount * bvec[0]
                self.wall_vels[idx][2*i+1] += blob_amount * bvec[1]
            
            
            
    def on_key_press(self, sym, mods):
        from pyglet.event import EVENT_HANDLED
        from pyglet.event import EVENT_UNHANDLED
        if self.finished:
            if self.season and not self.season.is_single_race():
                if not self.showing_season:
                    self.showing_season = True
                    self.season_head.show = True
                    self.set_summary(True)
                elif self.season.has_next_race():
                    #self.control.music.fadeout(1)        
                    self.control.switch_handler('race', racename=self.season.get_next_race(),
                                                ghost=None, race_mode=self.season.name,
                                                shipimg=self.race.player.svg_name, season=self.season,
                                                return_to=self.return_to)
                else:
                    self.control.music.fadeout(1)
                    self.control.switch_handler('menu', state=self.return_to)
            else:
                self.control.music.fadeout(1)
                self.control.switch_handler('menu', state=self.return_to)
            return EVENT_HANDLED
        if sym == pyglet.window.key.ESCAPE:
            self.race.cleanup()
            self.control.music.fadeout(1)
            self.control.switch_handler("menu", state=self.return_to)
            return EVENT_HANDLED
        else:
            return EVENT_UNHANDLED

    ## Rendering
    ############

    def on_resize(self, width, height):
        subwidth, subheight = width, height
        if WORLD_WIDTH * height > WORLD_HEIGHT * width:
            subheight = width * WORLD_HEIGHT // WORLD_WIDTH
        elif WORLD_WIDTH * height < WORLD_HEIGHT * width:
            subwidth = height * WORLD_WIDTH // WORLD_HEIGHT
        xoff = (width - subwidth) // 2
        yoff = (height - subheight) // 2
        # Remember these values for later.
        self.view_offset = (xoff, yoff)
        self.view_size = (subwidth, subheight)
        # Perform the normal ortho projection.
        glViewport(xoff, yoff, subwidth, subheight)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(0, subwidth, 0, subheight, -1, 1)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        return pyglet.event.EVENT_HANDLED



    def draw_minimap(self, pts_per_seg=10):
        xsize = self.view_size[0] / 4
        ysize = self.view_size[1] / 4
        if self.minimap_data:
            track, vdata, minx, miny, maxx, maxy = self.minimap_data
        else:
            track = []
            vdata = []
        cdata = []
        for n, seg in enumerate(self.race.track.segs):
            for i in xrange(pts_per_seg + (1 if n == len(self.race.track.segs) - 1 else 0)):
                t = i / pts_per_seg
                if not self.minimap_data:
                    track.append(seg.pos(t))
                    vdata.extend(seg.pos(t))
                d = abs(self.race.player.theta - n - t)
                if d > 5: d = 1
                else:
                    d = _ease(.2 * d)
                c = int(255 - 128 * d)
                cdata.extend((c,c,c))
        if not self.minimap_data:
            minx = min(x[0] for x in track)
            miny = min(x[1] for x in track)
            maxx = max(x[0] for x in track)
            maxy = max(x[1] for x in track)
        self.minimap_data = track, vdata, minx, miny, maxx, maxy

        xr = maxx - minx
        yr = maxy - miny
        scale = min(xsize / xr, ysize / yr)
        glLineWidth(scale * self.race.track.width)
        glPushMatrix()
        glTranslatef(self.view_size[0] * .85, self.view_size[1] *.85, 0)
        glScalef(scale, scale, scale)
        glTranslatef(-.5* (minx + maxx), -.5* (miny + maxy), 0)
        pyglet.graphics.draw(len(track), GL_LINE_STRIP, ('v2f', vdata), ('c3B', cdata))
        glColor3f(0,1,0)
        glPointSize(5)
        pyglet.graphics.draw(1, GL_POINTS, ('v2f', self.race.player.pos))
        glPopMatrix()
        glLineWidth(1)
  
    def draw_track(self):        
        drawables = []
        for n, w in enumerate(self.walls):
            drawables.append((n/WALL_SECTIONS, 'track', w+ (self.race.track.segs[n // WALL_SECTIONS].center, self.race.track.segs[n // WALL_SECTIONS].radius)))
        for e in self.race.entities:
            drawables.append((e.theta + 0.5, 'ent', e))
        if self.ghost_race:
            ghost = self.ghost_race.player
            drawables.append((ghost.theta + 0.4, 'ent', ghost))
        drawables.append((self.race.player.theta + 0.2, 'particles', None))
        drawables.append((1, 'duodenum', self.duodena[0]))
        drawables.append((len(self.race.track.segs) + 1, 'duodenum', self.duodena[1]))
        
        drawables.sort()
        
        sprite_vdata = (GLfloat * 8)(-1,-1,-1,1,1,1,1,-1)
        sprite_tdata = (GLfloat * 8)(0,0,0,1,1,1,1,0)
        
        #colors = [(1,0,0), (0,1,0), (0,0,1), (1,1,0)]
        vx, vy = self.race.player.pos
        for theta, kind, ddata in drawables:
            if kind == 'ent':
                e = ddata
                ex, ey = e.pos
                if (ex - vx) ** 2 + (ey - vy) ** 2 > WORLD_WIDTH ** 2: continue
                if e.svg_name == 'OBSTACLE':
                    phi = .75 + .5 * (self.race.track.pos(e.theta) - e.pos).length2 / self.race.track.width ** 2
                    if abs(theta - self.race.player.theta) > 3: glColor4f(phi,phi,phi,.5)
                    else: glColor4f(phi,phi,phi,1)
                    glEnable(GL_TEXTURE_2D)
                    glBindTexture(GL_TEXTURE_2D, self.obstacle_tex.id)
                    glPushMatrix()
                    glTranslatef(ex, ey, 0)
                    glScalef(e.radius * 1.5, e.radius * 1.5, 1)
                    glEnableClientState(GL_VERTEX_ARRAY)
                    glEnableClientState(GL_TEXTURE_COORD_ARRAY)
                    glVertexPointer(2, GL_FLOAT, 0, sprite_vdata)
                    glTexCoordPointer(2, GL_FLOAT, 0, sprite_tdata)
                    glDrawArrays(GL_QUADS, 0, 4)
                    glPopMatrix()
                else:    
                    svg = squirtle.SVG(data.image_path(e.svg_name), anchor_x='center', anchor_y='center')
                    if abs(theta - self.race.player.theta) > 3: glEnable(GL_TEXTURE_2D)
                    else: glDisable(GL_TEXTURE_2D)
                    svg.draw(ex, ey, angle=e.angle, scale=e.diameter/svg.height * 1.25)
            elif kind == 'track':
                idxs, vdata, tdata, pos, r = ddata
                x, y = pos
                if (x - vx) ** 2 + (y - vy) ** 2 > (WORLD_WIDTH + r) ** 2: continue
                if abs(theta - self.race.player.theta) > 3: glColor4f(.8, .8, .8, .5)
                else: glColor4f(1,1,1,1)
                #glColor3f(*(colors[int(theta) % len(colors)]))
                glEnable(GL_TEXTURE_2D)
                glBindTexture(GL_TEXTURE_2D, self.track_tex.id)
                glEnableClientState(GL_VERTEX_ARRAY)
                glEnableClientState(GL_TEXTURE_COORD_ARRAY)
                glVertexPointer(2, GL_FLOAT, 0, vdata)
                glTexCoordPointer(2, GL_FLOAT, 0, tdata)
                glDrawElements(GL_TRIANGLE_STRIP, len(idxs), GL_UNSIGNED_INT, idxs)
            elif kind == 'particles':
                glPointSize(self.view_size[0] // 20)
                self.particles.draw()
            elif kind == 'duodenum':
                idxs, vdata, tdata = ddata
                if abs(theta - self.race.player.theta) > 3: glColor4f(.8, .8, .8, .5)
                else: glColor4f(1,1,1,1)
                glEnable(GL_TEXTURE_2D)
                glBindTexture(GL_TEXTURE_2D, self.duodenum_tex.id)
                glEnableClientState(GL_VERTEX_ARRAY)
                glEnableClientState(GL_TEXTURE_COORD_ARRAY)
                glVertexPointer(2, GL_FLOAT, 0, vdata)
                glTexCoordPointer(2, GL_FLOAT, 0, tdata)
                glDrawElements(GL_QUADS, len(idxs), GL_UNSIGNED_INT, idxs)
                
        glDisable(GL_TEXTURE_2D)
                
        
    def on_draw(self):
        self.update_timer()
        
        glClearColor(0,0,0,1)
        self.window.clear()
        glEnable(GL_LINE_SMOOTH)
        glEnable(GL_BLEND)
        glColor3f(1,1,1)
        fx.draw_background(self.view_size, self.race.player.pos, self.cloud_tex)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        glPushMatrix()
        glTranslatef(self.view_size[0]/2,
                     self.view_size[1]/2,
                     0.0)
        glScalef(self.view_size[0] / WORLD_WIDTH,
                 self.view_size[1] / WORLD_HEIGHT,
                 1.0)
        glTranslatef(-self.race.player.pos.x,
                     -self.race.player.pos.y,
                     0.0)
        self.draw_track()
        #self.batch._update_draw_list() #Can probably be optimized, but this is easiest
        #self.batch.draw()
        #self.dummy_batch.draw()
        glPopMatrix()
        #fx.draw_background(self.view_size, self.race.player.pos, self.cloud_tex, True)
        self.draw_minimap()
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
        if self.fade_alpha:
            glColor4ub(0,0,0,self.fade_alpha)
            glBegin(GL_QUADS)
            glVertex2f(0,0)
            glVertex2f(self.view_size[0], 0)
            glVertex2f(self.view_size[0], self.view_size[1])
            glVertex2f(0, self.view_size[1])
            glEnd()
        self.root.draw()
        if DEBUG:
            self.fps.draw()
    def draw_racers(self):
        for ent in self.race.entities:
            self.ship_svg.draw(ent.pos.x, ent.pos.y, scale= ent.diameter / self.ship_svg.height, angle=ent.angle)

    ## Mode Methods
    ###############

    def connect(self, control):
        super(RaceMode, self).connect(control)
        self.window.set_mouse_visible(False)
        self.on_resize(*self.window.get_size())
        self.root.offset = self.view_offset
        self.root.size = self.view_size
        self.root.window = self.window
        self.update_position()
    def disconnect(self):
        self.root.window = None
        super(RaceMode, self).disconnect()

    def update_player(self):
        if self.interaction_mode == "play":
            accel = PLAYER_THRUST
            #This lot might be more naturally implemented with key-up/key-down?
            self.race.player.controls['thrust_rel'] = zero
            self.race.player.controls['rotation'] = 0
            self.race.player.controls['firing'] = False
            if self.keys[pyglet.window.key.UP]:
                self.race.player.controls['thrust_rel'] += v(accel, 0)
            if self.keys[pyglet.window.key.DOWN]:
                self.race.player.controls['thrust_rel'] += v(-accel/2, 0)
            if self.keys[pyglet.window.key.LEFT]:
                self.race.player.controls['rotation'] += PLAYER_ROTATION
            if self.keys[pyglet.window.key.RIGHT]:
                self.race.player.controls['rotation'] -= PLAYER_ROTATION
            if self.keys[pyglet.window.key.SPACE]:
                self.race.player.controls['firing'] = True

    def update_timer(self):
        if self.stopping:
            self.timer.update(self.race.player_finish_time / 100 / TICK_RATE)
        else:
            self.timer.update(self.race.current_tick / TICK_RATE)
    
        
    def update_walls(self):
        for n, w in enumerate(self.walls):
            if self.dead_walls[n]: continue
            wv = self.wall_vels[n]
            cw = self.correct_walls[n]
            wp = self.wall_vxs[n]
            dead = True
            for i in xrange(len(cw)):
                wa = .02 * (cw[i] - wp[i])
                wv[i] = (wv[i] + wa) * 0.95
                wp[i] += wv[i]
                if -1e-2 < wa < 1e-2 and -1e-1 < wv[i] < 1e-1:
                    wv[i] = 0 
                    wp[i] = cw[i]
                else:
                     dead = False
            self.dead_walls[n] = dead
            if dead: wp = cw
            self.walls[n][1][:] = wp

            
    def cosmetic_updates(self):
        self.update_walls()
        for ent in self.race.entities:
            if ent.particles and ent.controls['thrust_rel'] != zero:
                p = [v(-15, 15), v(-15, -15)][self.race.current_tick % 2]
                p += v(0, random.gauss(0,5))
                self.particles.add_particle(ent.pos + p.rotated(ent.angle), .5*ent.velocity + v(random.gauss(0,.5), random.gauss(0,.5)))
                
            
    def tick(self):
        
        if self.countdown.color[3] > 0:
            self.countdown.color = self.countdown.color[0:3] + (max(self.countdown.color[3] - 4, 0),)
            self.countdown.clean_content()
            if 0 < self.countdown_ticks < 60:
                self.track_name.color = (255, 255, 255, max(self.countdown.color[3] - 4, 0),)
                self.track_fact.color = (255, 255, 255, max(self.countdown.color[3] - 4, 0),)
                self.track_name.clean_content()
                self.track_fact.clean_content()
                
        if not self.started:
            
            self.countdown_ticks -= 1
            if self.countdown_ticks % 60 == 0:
                self.countdown.text = ['GO!', '1', '2', '3'][self.countdown_ticks // 60]
                self.countdown.color = (255, 255, 255, 255)
                if self.countdown_ticks == 0:
                    sound.highbeep()
                else:
                    sound.lowbeep()
            if self.countdown_ticks == 0:
                self.track_name.show = False
                self.track_fact.show = False
                self.started = True
        
                
        elif not self.stopping or self.fade_alpha < 255:
            self.particles.tick()
            
            self.update_player()
            self.cosmetic_updates()
            self.race.tick()
            if self.race.current_tick == 1:
                if self.control.music.song != 'rephlektor.mp3':
                    self.control.music.change_song('rephlektor.mp3', fadein=1.0)
            if self.ghost_race is not None:
                self.ghost_race.tick()

            if not self.stopping:
                ppos = self.race.player_position
                if self.position.alpha > 127: 
                    self.position.alpha -= 2
                    self.position.clean_content()
                if self.position.pos != ppos:
                    self.update_position()
                if self.race.is_victorious():
                    #Win!
                    self.stopping = True
                    self.set_summary()
               
            else:
                self.fade_alpha += 3
                for s in self.summary_pos, self.summary_name, self.summary_time:
                    s.color = (255, 255, 255, self.fade_alpha)
                    s.clean_content()
                self.position.alpha = max(self.position.alpha - 3, 0)
                self.position.clean_content()
                self.timer.alpha = max(self.timer.alpha - 3, 0)
                self.timer.clean_content()
                if self.fade_alpha == 255:
                    self.race.cleanup()
                    if self.ghost_race is not None:
                        self.ghost_race.cleanup()
                    if self.interaction_mode == "play":
                        hiscores.encode_ghost(self.race)
                        hiscores.set_personal_score(self.race)
                        hiscores.set_global_score(self.control.rpc, self.race)
                    if self.season is not None and not self.season.is_single_race():
                        self.season.update_scores(self.race.finishers)
                    self.finished = True
        if self.started and self.countdown.color[3] == 0:
            self.countdown.show = False

    def update_position(self):
        self.position.nracers = len(self.race.racers)
        self.position.pos = ppos = self.race.player_position
        self.position.alpha = 255
        self.position.clean_content()


    def set_summary(self, season=False):
        ordinals = ["1st", "2nd", "3rd", "4th", "5th", "6th", "7th", "8th", "9th", "10th", "11th", "12th", "13th", "14th", "15th", "16th"]
        if season:
            self.summary_pos.text = ""
            self.summary_name.text = ""
            self.summary_time.text = ""
            for n, data in enumerate(self.season.standings):
                racer, score = data
                self.summary_pos.text += ordinals[n] + '\n'
                try:
                    self.summary_name.text += racer.pilot_name + '\n'
                except TypeError:
                    self.summary_name.text += config.username + '\n'
                self.summary_time.text += '%d points\n' % score
        else:
            for n, data in enumerate(self.race.finishers):
                racer, time = data
                if len(self.race.finishers) > 1:
                    self.summary_pos.text += ordinals[n] + '\n'
                self.summary_name.text += racer.pilot_name + '\n'
                self.summary_time.text += formatscore(time) + '\n'
        for s in self.summary_pos, self.summary_name, self.summary_time:
            s.clean_content()
        
            #txt += "%s %s %s\n" % (ordinals[n], racer.pilot_name, formatscore(time))
        #self.summary.text = txt
