from shared import *

from google.appengine.ext import webapp
from google.appengine.ext.webapp.util import run_wsgi_app


## Handler
##########

class RPCHandler(webapp.RequestHandler):

    func = None

    def validate(self):
        # Deserialise 'data' parameter.
        data = self.request.get('data', None)
        if data is not None:
            try:
                result = deserialise(data)
            except DeserialiseError:
                raise RPCError(RPC_CANNOT_DESERIALISE, 'input')
        else:
            raise RPCError(RPC_CANNOT_DESERIALISE, 'noinput')
        # Validate 'data' parameter.
        if not isinstance(result, dict):
            raise RPCError(RPC_CANNOT_DESERIALISE, 'badinput')
        # Validate 'version' argument.
        if 'version' not in result:
            raise RPCError(RPC_MISSING_ARGUMENT, 'version')
        elif not isinstance(result['version'], list):
            raise RPCError(RPC_INVALID_ARGUMENT, 'version')
        elif tuple(result['version']) > application_data[0]:
            raise RPCError(RPC_CLIENT_TOO_NEW)
        elif tuple(result['version']) < application_data[1]:
            raise RPCError(RPC_CLIENT_TOO_OLD)
        # Validate 'devel' argument.
        if 'devel' not in result:
            raise RPCError(RPC_MISSING_ARGUMENT, 'devel')
        elif not isinstance(result['devel'], bool):
            raise RPCError(RPC_INVALID_ARGUMENT, 'devel')
        elif result['devel'] != application_data[2]:
            raise RPCError(RPC_INVALID_CLIENT)
        # Validate 'args' argument.
        if 'args' not in result:
            raise RPCError(RPC_MISSING_ARGUMENT, 'args')
        elif not isinstance(result['args'], list):
            raise RPCError(RPC_INVALID_ARGUMENT, 'args')
        # Validate 'kwds' argument.
        if 'kwds' not in result:
            raise RPCError(RPC_MISSING_ARGUMENT, 'kwds')
        elif not isinstance(result['kwds'], dict):
            raise RPCError(RPC_INVALID_ARGUMENT, 'kwds')
        # Return 'args' and 'kwds' arguments.
        return result['args'], result['kwds']

    def respond(self):
        if False:
            print "Content-Type: text/plain"
            print
            print "DEBUG"
        try:
            try:
                # Validate parameters.
                args, kwds = self.validate()
                # Call the RPC function.
                try:
                    result = self.func(*args, **kwds)
                except BaseException, exc:
                    message = "%s: %s" % (type(exc).__name__, exc)
                    raise RPCError(RPC_UNKNOWN_ERROR, message)
                # Serialise the result in JSON.
                try:
                    output = serialise(result)
                except SerialiseError:
                    raise RPCError(RPC_CANNOT_SERIALISE, 'output')
                # Return the JSON result.
                self.response.headers['Content-Type'] = 'text/plain'
                self.response.out.write(output)
            # Managed exceptions.
            except RPCError, exc:
                self.response.set_status(500, exc.serialise())
        except Exception, exc:
            import sys, traceback
            output = traceback.format_exception(*sys.exc_info())
            error = RPCError(RPC_UNKNOWN_ERROR, "".join(output))
            self.response.set_status(500, error.serialise())

    get = post = respond


## Utilities
############

application_registry = {}
application_data = [None, None, False]

def makerpc(func):
    cls = type(func.func_name, (RPCHandler,), {"func" : func})
    application_registry["/%s" % func.func_name] = cls
    return cls

def start(version, compat=None, debug=False):
    version = tuple(version)
    compat = tuple(compat or version)
    debug = bool(debug)
    if application_data == [None, None, False]:
        application_data[:] = [version, compat, debug]
    if application_data == [version, compat, debug]:
        registry = sorted(application_registry.items())
        application = webapp.WSGIApplication(registry, debug=debug)
        run_wsgi_app(application)


## Defaults
###########

@makerpc
def server_index(handler):
    return sorted(cls.__name__ for cls in application_registry.values())

@makerpc
def server_version(handler):
    return application_data[0]

@makerpc
def server_compat(handler):
    return application_data[1]

@makerpc
def server_debug(handler):
    return application_data[2]
