try:
    import json
except:
    import simplejson as json

## Errors
#########

RPC_CANNOT_SERIALISE = 0
RPC_CANNOT_DESERIALISE = 1
RPC_MISSING_ARGUMENT = 2
RPC_INVALID_ARGUMENT = 3
RPC_CLIENT_TOO_NEW = 4
RPC_CLIENT_TOO_OLD = 5
RPC_INVALID_CLIENT = 6
RPC_CALLBACK_ERROR = 7
RPC_SOCKET_TIMEOUT = 8
RPC_SOCKET_ERROR = 9
RPC_HTTP_ERROR = 10
RPC_UNKNOWN_ERROR = 11

_revmap = dict((v, k) for k, v in globals().items() if k.startswith("RPC_"))

class RPCError(Exception):

    def __init__(self, errno, errmsg=None):
        self.errno = errno
        self.errmsg = errmsg
        self.errname = _revmap[errno]

    def serialise(self):
        return serialise((self.errno, self.errmsg))

    @classmethod
    def deserialise(cls, data):
        errno, errmsg = deserialise(data)
        return RPCError(errno, errmsg)

class SerialiseError(Exception):
    pass

class DeserialiseError(Exception):
    pass


## Serialisation
################

def serialise(obj):
    try:
        return json.dumps(obj)
    except:
        raise SerialiseError

def deserialise(str):
    if isinstance(str, unicode):
        str = str.encode('utf-8')
    try:
        return json.loads(str)
    except:
        raise DeserialiseError
