from __future__ import division

import random

from common import *
from constants import *

from model.enemy import enemy_racers
from model.player import Player

import presidents

place_points = [100, 75, 50, 25, 10, 5, 2, 0]

class Season(object):

    def __init__(self, name, single=False):
        assert name in presidents.seasons
        self.name = name
        self.races = presidents.seasons[name][:]
        self.single = single
        
        season_rng = random.Random(hash(name) & 0xffffffff)

        racers = enemy_racers[:]
        self.racers = []
        num_racers = 7
        while num_racers > 0:
            season_rng.shuffle(racers)
            self.racers.append(racers.pop())
            num_racers -= 1
        
        self.scores = {}
        for r in self.racers:
            self.scores[r] = 0
            self.scores[Player] = 0
            
    def has_next_race(self):
        return (not self.single) and len(self.races) > 0
            
    def get_next_race(self):
        return self.races.pop(0)

    def is_single_race(self):
        return self.single
        
    def update_scores(self, finishers):
        for idx, f in enumerate(finishers):
            racer, time = f
            self.scores[racer.__class__] += place_points[idx]

    @property            
    def standings(self):
        res = map(lambda r: (r, self.scores[r]), [Player] + self.racers)
        res.sort(key=lambda p: p[1], reverse=True)
        return res